
use v6;
use Perl::Meta::Class::Behavior;

class Perl::Meta::MetaClass is ::Perl::Meta::Class::Behavior;

=pod

=head1 NAME

Perl::Meta::MetaClass - A meta-meta-model for Perl Classes

=head1 SYNOPSIS

=head1 DESCRIPTION

=head1 SUPERCLASS

=over 4

=item B<Perl::Meta::Class::Behavior>

=back

=head1 METHODS

=head1 SEE ALSO

=head1 AUTHORS

Stevan Little E<lt>stevan@iinteractive.comE<gt>

=cut
